package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerPackageController
import com.aitrainer.api.model.*
import com.aitrainer.api.repository.*
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class AppCustomerPackageTest {

    @Autowired
    private lateinit var customerRepository: CustomerRepository

    @Autowired
    private lateinit var customerExerciseDeviceRepository: CustomerExerciseDeviceRepository

    @Autowired
    private lateinit var exercisesRepository: ExercisesRepository

    @Autowired
    private lateinit var productTestRepository: ProductTestRepository

    @Autowired
    private lateinit var purchaseRepository: PurchaseRepository

    @Autowired
    private lateinit var customerPropertyRepository: CustomerPropertyRepository

    @Autowired
    private lateinit var exerciseResultRepository: ExerciseResultRepository

    @Test
    fun customerPackageTest() {
        val gson = Gson()

        val controller = CustomerPackageController(customerRepository, customerExerciseDeviceRepository,
            exercisesRepository, productTestRepository, purchaseRepository, customerPropertyRepository,
            exerciseResultRepository)
        var response: ResponseEntity<*> = controller.getCustomerPackageData(91)
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)

        response = controller.getCustomerPackageData(90)
        assertEquals(response.statusCode, HttpStatus.OK)

        val appPackageJson: String = response.body as String
        assertTrue(appPackageJson.isNotEmpty())


        val packages = appPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            if (record[0] == Customer::class.simpleName) {
                val customerJson: String = record[1]
                val type = object : TypeToken<Customer?>() {}.type
                val customer: Customer = gson.fromJson(customerJson, type)
                assertEquals(customer.email, "sw@andio.biz")
                assertEquals(customer.birthYear, 1972)
            } else if (record[0] == Exercises::class.simpleName) {
                val exercisesJson: String = record[1]
                val type = object : TypeToken<List<Exercises?>?>() {}.type
                val exerciseList: List<Exercises> = gson.fromJson(exercisesJson, type)
                assertTrue(exerciseList.isNotEmpty())
                assertEquals(exerciseList[0].exerciseTypeId, 37)
                assertEquals(exerciseList[0].quantity, 40.0)
            } else if (record[0] == CustomerExerciseDevice::class.simpleName) {
                val exerciseDeviceJson: String = record[1]
                val type = object : TypeToken<List<CustomerExerciseDevice?>?>() {}.type
                val customerExerciseDeviceList: List<CustomerExerciseDevice> = gson.fromJson(exerciseDeviceJson, type)
                assertTrue(customerExerciseDeviceList.isNotEmpty())
                assertEquals(customerExerciseDeviceList[1].exerciseDeviceId,2)
            } else if (record[0] == ProductTest::class.simpleName) {
                val productTestJson: String = record[1]
                val type = object : TypeToken<List<ProductTest?>?>() {}.type
                val productTestList: List<ProductTest> = gson.fromJson(productTestJson, type)
                assertTrue(productTestList.isNotEmpty())
                assertEquals(productTestList[0].productId,1)
            } else if (record[0] == CustomerProperty::class.simpleName) {
                val propertyJson: String = record[1]
                val type = object : TypeToken<List<CustomerProperty?>?>() {}.type
                val propertyList: List<CustomerProperty> = gson.fromJson(propertyJson, type)
                assertTrue(propertyList.isNotEmpty())
                assertEquals(propertyList[1].propertyId,1)
                assertEquals(propertyList[1].propertyValue,82.0)
            }
        }

    }

}